#include <iostream>
#include <iomanip>
#include <fstream>
#include <cmath>
#include <stdio.h>
#include <stdlib.h>
#include <string>

using namespace std;

int main(void)
{
	cout << "Reading parameters for runoff simulation." << endl;
	ifstream finParam("../param.dat");
	if(!finParam){
		cout << "param.dat cannot open." << endl;
		return 1;
	}

	// vZ̈ simulation range
	int wCol,wRow, wTopoScale;
	double wSWLON, wSWLAT;

	finParam >> wTopoScale;
	cout << "  Topography scale : " << wTopoScale << endl;

	finParam >> wCol; 
	finParam >> wRow; 
	cout << "  Number of col : " << wCol << "  Number of row : " << wRow << endl;

	finParam >> wSWLON; 
	finParam >> wSWLAT;
	cout << " Sorth West corner Longitude (degree) : " << wSWLON 
		 << " Latitude (deg) : " << wSWLAT << endl;

	const int Col = wCol;     // column number of study area
	const int Row = wRow;     // row number of study area
	const double SWLON = wSWLON; // South west corner of longitude (degree)
	const double SWLAT = wSWLAT; // South west corner of latiitude (degree)

	// ̓f[^̈ input data range
	cout << "2) Reading parameters for input data." << endl;

	int wROFnum_row, wROFnum_col;
	finParam >> wROFnum_col;
	finParam >> wROFnum_row;
	cout << "  Number of col : " << wROFnum_col 
		 << "  Number of row : " << wROFnum_row << endl;

	int wpO_x, wpO_y;
	finParam >> wpO_x;
	finParam >> wpO_y;
	cout << "  Origin column number : " << wpO_x 
		 << " row number : " << wpO_y << endl;

	float wROF_WLON;
	finParam >> wROF_WLON; 
	cout << "  The longitude of the center of input grid at the southwest corner : " 
		<< wROF_WLON << endl;

	float wROF_WLAT;
	finParam >> wROF_WLAT; 
	cout << "  The latitude of the center of input grid at the southwest corner : " 
		<< wROF_WLAT << endl;

	int wInputData_Time_Step;
	finParam >> wInputData_Time_Step; 
	cout << "  time interval of input data [sec] : " 
		 << wInputData_Time_Step << endl;
	
	const int ROFnum_col = wROFnum_col;
	const int ROFnum_row = wROFnum_row; 
	const int pO_x = wpO_x;
	const int pO_y = wpO_y;
	const float ROF_WLON = wROF_WLON;
	const float ROF_WLAT = wROF_WLAT;
	const int InputData_Time_Step = wInputData_Time_Step;

	// vZ simulation term
	cout << "3) Reading parameters for simuration term." << endl;
	int wstartYear, wendYear, wstartMonth, wendMonth;
	finParam >> wstartYear;
	finParam >> wendYear;
	finParam >> wstartMonth;
	finParam >> wendMonth;
	cout << "  Start year : " << wstartYear << "  Start month : " << wstartMonth << endl;
	cout << "  End year : " << wendYear << "  End month : " << wendMonth << endl;

	const int startYear = wstartYear; // vZJnN start year
	const int endYear   = wendYear; // vZIN end year

	if (startYear > endYear) {
		cout << "Simulation year setting is wrong" << endl;
		exit(1);
	}

	const int startMonth = wstartMonth;   // vZJn start month
	const int endMonth   = wendMonth;  // vZI end month

	finParam.close();
	//--------------------- end parameter reading ------------------------------------------------

	const int dayMonth[] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

	// setting column and row number
	cout << "Input column number to extract data!" << endl;
	cout << "Column number begins from 1 from east." << endl;
	cin >> wCol;

	string ss;
	if (wCol > Col || wCol < 0) {
		cout << "Column number is out of range!" << endl;
		return 1;
	}

	cout << "Input row number to extract data!" << endl;
	cout << "Row number begins from 1 from south." << endl;
	cin >> wRow;
	if (wRow > Row || wRow < 0) {
		cout << "Row number is out of range!" << endl;
		return 1;
	}

	cout << "Column number : " << wCol << " Row number : " << wRow << endl;

	int iyear, imonth;
	cout << "Input year!" << endl;
	cin >> iyear;
	cout << "Input month!" << endl;
	cin >> imonth;

	int max_time_counter = dayMonth[imonth-1];
	float discharge;
	char DischargeFileName[256];
	ifstream FileDaily; // Daily data

	//sprintf_s(DischargeFileName, 256, "197912_dailyDischarge.bin");
	sprintf(DischargeFileName, "./%04d%02d_dailyDischarge.bin", iyear, imonth);

	cout << "Read " << DischargeFileName << endl;
	FileDaily.open(DischargeFileName, ios::in | ios::binary);

	if(!FileDaily){
		cout << "Daily discharge data cannot open." << endl;
		return 1;
	}
	ofstream outFile;
	//sprintf_s(DischargeFileName, 256, "./%d-%d_discharge.dat", wCol, wRow);
	sprintf(DischargeFileName, "./%04d%02d-%d-%d_discharge.dat", iyear, imonth, wCol, wRow);
	cout << "Write " << DischargeFileName << "\r";

	outFile.open(DischargeFileName);
	if(!outFile){
		cout << "Time series discharge output file cannot open." << endl;
		return 1;
	}

	for (int i = 1; i <= max_time_counter; i++) {
		for (int j = 1; j <= Row; j++) {
			for (int k = 1; k <= Col; k++) {
				FileDaily.read((char*)&discharge, sizeof(float));
				if (j == wRow && k == wCol) {
					outFile << iyear << " " << imonth << " " 
							<< i << " " << discharge << endl;
				}
			}
		}

	}

	FileDaily.close();
	outFile.close();
	
}

